#include "prefix.h"

#include "anglecontrol.h"
#include "winmessage.h"
#include "dc.h"
#include "gdiobject.h"
#include "string.h"
#include "resstring.h"
#include "pi.h"
#include "verify.h"

#include <math.h>

/*************************************************

	Rg[̃NX

**************************************************/

APP_BEGIN_NAMESPACE

AngleControl::AngleControl()
:m_angle(0)
,m_isAsRotation(false)
{
	
}

AngleControl::~AngleControl() throw()
{
}

LRESULT AngleControl::WindowProc(UINT msg,WPARAM wparam,LPARAM lparam)
{
	switch(msg) {
	case WM_LBUTTONDOWN:
		OnLButtonDown((short)LOWORD(lparam),(short)HIWORD(lparam));
		break;

	case WM_MOUSEMOVE:
		OnMouseMove((short)LOWORD(lparam),(short)HIWORD(lparam));
		break;

	case WM_LBUTTONUP:
		OnLButtonUp();
		break;

	case WM_GETDLGCODE:
		return OnGetDlgCode();

	case APP_WM_REFLECT_DRAWITEM:
		OnDrawItem((DRAWITEMSTRUCT*)wparam);
		break;

	default:
		return Window::WindowProc(msg,wparam,lparam);
	}
	return 0;
}

//\
void AngleControl::OnDrawItem(DRAWITEMSTRUCT *drawItemStruct) const
{
	DrawItemDC dc(drawItemStruct->hDC);
	RECT rect(drawItemStruct->rcItem);

	bool isDisabled=(drawItemStruct->itemState & ODS_DISABLED)!=0;

	int backColor=COLOR_WINDOW;
	if(isDisabled) {
		backColor=COLOR_3DFACE;
	}
	HBRUSH hFrameBrush(::GetSysColorBrush(backColor));

	GdiObject frameBrush(hFrameBrush);
	dc.SelectObject(&frameBrush);
	VERIFY(::Ellipse(dc.Get(),rect.left,rect.top,rect.right,rect.bottom));
	dc.DeselectObject(OBJ_BRUSH);

	if(!isDisabled) {
		DrawArrow(dc.Get(),rect,m_angle);
		DrawNumber(dc.Get(),rect,m_angle);
	}
}

#define COUNT 4
static double s_arrow[COUNT*2]={
	0.9,0,
	0.5,0.2,
	0.5,-0.2,
	0.9,0
};

//̐F
#define ARROWCOLOR	0xffff00

//`
void AngleControl::DrawArrow(HDC hdc,const RECT &rect,double angle) const
{
	DrawItemDC dc(hdc);

	double rx=Width(rect)/2.0;
	double ry=Height(rect)/2.0;

	double angleX=cos(angle);
	double angleY=sin(angle);

	POINT points[COUNT];
	FOR(int i=0;i<COUNT;++i) {
		points[i].x=static_cast<int>
			((angleX*s_arrow[i*2]+angleY*s_arrow[i*2+1])*rx+rx);
		points[i].y=static_cast<int>
			((angleX*s_arrow[i*2+1]-angleY*s_arrow[i*2])*ry+ry);
	}

	GdiObject brush(::CreateSolidBrush(ARROWCOLOR));
	dc.SelectObject(&brush);

	VERIFY(::Polygon(dc.Get(),points,COUNT));
}

//px̐l
void AngleControl::DrawNumber(HDC hdc,const RECT &rect,double angle) const
{
	DrawItemDC dc(hdc);
	dc.SetBkMode(TRANSPARENT);

	double dir360=angle*360/(PI*2);
	if(!m_isAsRotation) {
		dir360+=360;
	}
	dir360=fmod(dir360,360);

	String string;
	string.Format(IDS_FORMAT_ANGLE,dir360);
	VERIFY(::DrawText(dc.Get(),string.Get(),string.Length()
		,const_cast<RECT*>(&rect),DT_CENTER|DT_SINGLELINE|DT_VCENTER)>0);
}

//󂯕tL[͂̎
UINT AngleControl::OnGetDlgCode() const
{
	return 0;
}

//{^
void AngleControl::OnLButtonDown(int x,int y)
{
	Change(x,y);
	SetCapture();
}

//}EX𓮂
void AngleControl::OnMouseMove(int x,int y)
{
	if(::GetCapture()==GetHWND()) {
		Change(x,y);
	}
}

//{^𗣂
void AngleControl::OnLButtonUp()
{
	if(::GetCapture()==GetHWND()) {
		::ReleaseCapture();
		::PostMessage(GetParent(),APP_WM_CONTROL
			,MAKELONG(GetID(),ANGLE_CHANGED)
			,0);
	}
}

//ړ
void AngleControl::Change(int x,int y)
{
	RECT rect;
	GetClientRect(&rect);

	int hCenter=Width(rect)/2;
	int vCenter=Height(rect)/2;

	m_angle=APP_ATAN_XY(x-hCenter,vCenter-y);

	InvalidateRect();
}

//݂̕𓾂
double AngleControl::GetAngle() const
{
	return m_angle;
}

//ݒ
void AngleControl::SetAngle(double angle)
{
	m_angle=angle;
	InvalidateRect();
}

//]pƂĕ\
void AngleControl::SetAsRotation(bool isAsRotation)
{
	m_isAsRotation=isAsRotation;
}

void AngleControl::OnDestroy()
{
	UnsubclassWindow();
	Window::OnDestroy();
}

APP_END_NAMESPACE