#include "prefix.h"

#include "appdoc.h"
#include "core.h"
#include "winman.h"
#include "image.h"
#include "channel.h"
#include "history.h"
#include "historywriter.h"
#include "historyreader.h"
#include "verify.h"

#include "delaybuffer.h"
#include "delaydiff.h"

#ifndef APP_STANDALONE
#include "pixia.h"
#endif

#include <windows.h>

/*************************************************

	AvP[V NX

**************************************************/

APP_BEGIN_NAMESPACE

class AppDocMember
{
public:
	Core core;	//ۂɗqēƂ
	Image image;	//摜obt@
	Image backupImage;
	SIZE size;	//̑傫
	History history;	//
};

AppDoc::AppDoc()
:m_m(new AppDocMember)
{
}

AppDoc::~AppDoc() throw()
{
}

//摜̑傫
SIZE AppDoc::GetSize() const
{
	return m_m->size;
}

//VJ
void AppDoc::OnNewDocument()
{
#ifdef APP_STANDALONE
	m_m->size.cx=300;
	m_m->size.cy=200;
#else
	m_m->size=PixiaSize();
#endif
	m_m->core.SetSize(m_m->size);
	m_m->core.Initialize();

	m_m->image.Create(m_m->size,PIXEL_BYTES);
	memset(m_m->image.GetW(),0,m_m->image.TotalBytes());
#ifndef APP_STANDALONE
	PixiaGetImage(&(m_m->image));
#endif
	m_m->backupImage.Dup(m_m->image);
}

Core * AppDoc::GetCore()
{
	return &(m_m->core);
}

Image *AppDoc::GetImage()
{
	return &(m_m->image);
}

void AppDoc::OnCloseDocument()
{
#ifndef APP_STANDALONE
	PixiaSetImage(&(m_m->image));
#endif
}

//ɖ߂ł邩
bool AppDoc::CanUndo() const
{
	return m_m->history.CanUndo();
}

//蒼ł邩
bool AppDoc::CanRedo() const
{
	return m_m->history.CanRedo();
}

#define HISTORY_DOC	'Docu'
#define HISTORY_IMAGE 'Imag'

//ɓo^
void AppDoc::Record()
{
	assert(!m_m->core.HasParticle());

	HistoryWriter writer(&(m_m->history));
	if(writer.BeginWrite()) {
		DelayBuffer delayImage(m_m->image.Get(),m_m->image.TotalBytes());
		DelayBuffer delayBackup(m_m->backupImage.Get(),m_m->backupImage.TotalBytes());

		DelayDiff delayDiff(&delayImage,&delayBackup,false);

		writer.WriteData(&delayDiff);

		if(delayDiff.HasDifference()) {
			writer.Push(HISTORY_DOC,HISTORY_IMAGE);
		}

		writer.EndWrite();
	}

	m_m->backupImage.Dup(m_m->image);
}

//ɖ߂
void AppDoc::Undo()
{
	assert(m_m->history.CanUndo());
	UndoRedo(true);
}

//蒼
void AppDoc::Redo()
{
	assert(m_m->history.CanRedo());
	UndoRedo(false);
}

void AppDoc::UndoRedo(bool undo)
{
	HistoryReader reader(&(m_m->history));
	reader.BeginRead(undo);

	for(;!reader.IsEnd();reader.Next()) {
		assert(reader.GetKey()==HISTORY_DOC);
		int key=reader.GetSubKey();
		if(key==HISTORY_IMAGE) {
			DelayBuffer delayImage(m_m->image.Get(),m_m->image.TotalBytes());

			std::auto_ptr<Delay> data;
			reader.ReadData(&data);

			DelayDiff diff(&delayImage,data.get(),!undo);

			VERIFY(diff.Read(m_m->image.GetW(),m_m->image.TotalBytes())
				==m_m->image.TotalBytes());

			m_m->backupImage.Dup(m_m->image);

		} else {
			assert(false);
		}
	}
	reader.EndRead();
}

//ݒύX
void AppDoc::OnChangeOptions()
{
	m_m->history.OnChangeOptions();
}

APP_END_NAMESPACE
