#include "prefix.h"

#include "appframe.h"
#include "apptoolbar.h"
#include "toolbar.h"
#include "registry.h"

#include "exception.h"
#include "fileexception.h"
#include "error.h"

#include "command.h"
#include "aboutdialog.h"
#include "string.h"
#include "main.h"
#include "winman.h"
#include "idtovalue.h"
#include "verify.h"

#include "rescommand.h"
#include "resstring.h"

#include <windows.h>
#include <commctrl.h>
#include <shellapi.h>
#include <stdlib.h>
#include <tchar.h>

/*************************************************

	AvP[V Ct[EBhE

**************************************************/

APP_BEGIN_NAMESPACE

AppFrame::AppFrame()
:m_toolBar(NULL)
,m_statusBar(NULL)
{
}

AppFrame::~AppFrame() throw()
{
}

void AppFrame::Create(HWND parentWindow)
{
	Frame::Create(m_initWindowRect,parentWindow);
	int showMode;
	if(m_initMaximize) {
		showMode=SW_SHOWMAXIMIZED;
	} else {
		showMode=SW_SHOWNORMAL;
	}
	ShowWindow(showMode);

	m_statusBar=::CreateStatusWindow(
		WS_CHILD
		,NULL,GetHWND(),ID_STATUSBAR);
	if(m_statusBar==NULL) {
		throw Exception(APPERR_CANT_CREATE_WINDOW);
	}
	if(m_initShowStatusBar) {
		::ShowWindow(m_statusBar,SW_SHOW);
	}

	m_toolBar=CreateToolBar(g_toolBarButtons
		,ID_TOOLBAR,GetHWND());
	if(m_initShowToolBar) {
		::ShowWindow(m_toolBar,SW_SHOW);
	}

	AdjustChildWindows();
}

//AvP[V̏
void AppFrame::AppInit(Registry *registry)
{
	ExchangeSettings(registry);
}

//ݒ̕ۑ
void AppFrame::AppSave(Registry *registry)
{
	ExchangeSettings(registry);
}

//AvP[V̏I
void AppFrame::AppQuit() throw()
{
}

//WXgƐݒ
void AppFrame::ExchangeSettings(Registry *registry)
{
	registry->OpenSubKey(_T("Frame"));

	registry->Exchange(_T("ShowToolBar"),&m_initShowToolBar,true);
	registry->Exchange(_T("ShowStatusBar"),&m_initShowStatusBar,true);

	RECT defaultRect={
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		CW_USEDEFAULT
	};

	registry->Exchange(_T("Window"),&m_initWindowRect,defaultRect);
	registry->Exchange(_T("Maximize"),&m_initMaximize,false);
}

//EBhEOɏԂϐɑޔ
void AppFrame::OnClose()
{
	m_initShowToolBar=::IsWindowVisible(m_toolBar)!=FALSE;
	m_initShowStatusBar=::IsWindowVisible(m_statusBar)!=FALSE;

	WINDOWPLACEMENT windowPlacement;
	memset(&windowPlacement,0,sizeof(WINDOWPLACEMENT));
	windowPlacement.length=sizeof(WINDOWPLACEMENT);
	::GetWindowPlacement(GetHWND(),&windowPlacement);
	m_initWindowRect=windowPlacement.rcNormalPosition;
	m_initMaximize=windowPlacement.showCmd==SW_SHOWMAXIMIZED;
	Frame::OnClose();
}

//R}h
void AppFrame::OnCommand(int id,int notifyCode,HWND hwnd)
{
	switch(id) {
	case ID_VIEW_TOOLBAR:
		OnViewToolBar();
		break;

	case ID_VIEW_STATUSBAR:
		OnViewStatusBar();
		break;

	case ID_SHOW_HELP:
		OnShowHelp();
		break;

	case ID_SHOW_ABOUT:
		OnShowAbout();
		break;

	case ID_VIEW_GENERAL:
	case ID_VIEW_SPRAY:
	case ID_VIEW_COLOR:
	case ID_VIEW_CATALOG:
		OnViewPanel(id);
		break;

	default:
		Frame::OnCommand(id,notifyCode,hwnd);
		break;
	}
}

//R}hGUI̍XV
void AppFrame::OnUpdateCommand(int id,Command *command)
{
	switch(id) {
	case ID_VIEW_TOOLBAR:
		OnUpdateViewToolBar(command);
		break;

	case ID_VIEW_STATUSBAR:
		OnUpdateViewStatusBar(command);
		break;

	case ID_VIEW_GENERAL:
	case ID_VIEW_SPRAY:
	case ID_VIEW_COLOR:
	case ID_VIEW_CATALOG:
		OnUpdateViewPanel(id,command);
		break;

	default:
		Frame::OnUpdateCommand(id,command);
		break;
	}
}

//c[o[̕\ؑ
void AppFrame::OnViewToolBar()
{
	ToggleShowWindow(m_toolBar);
	AdjustChildWindows();
}

void AppFrame::OnUpdateViewToolBar(Command *command)
{
	bool isVisible=::IsWindowVisible(m_toolBar)!=FALSE;
	command->SetCheck(isVisible);
}

//Xe[^Xo[̕\ؑ
void AppFrame::OnViewStatusBar()
{
	ToggleShowWindow(m_statusBar);
	AdjustChildWindows();
}

void AppFrame::OnUpdateViewStatusBar(Command *command)
{
	command->SetCheck(::IsWindowVisible(m_statusBar)!=FALSE);
}

//EBhE̕\؂ւ
void AppFrame::ToggleShowWindow(HWND hwnd)
{
	if(::IsWindowVisible(hwnd)) {
		::ShowWindow(hwnd,SW_HIDE);
	} else {
		::ShowWindow(hwnd,SW_SHOW);
	}
}

//o[W
void AppFrame::OnShowAbout()
{
	AboutDialog dialog;
	dialog.DoModal(GetHWND());
}

//̕\
void AppFrame::OnShowHelp()
{
	String help;
	VERIFY(help.LoadString(IDS_HELPFILE));

	_TCHAR moduleName[_MAX_PATH];
	::GetModuleFileName(GetInstance(),moduleName,_MAX_PATH);
	_TCHAR drive[_MAX_DRIVE];
	_TCHAR dir[_MAX_DIR];
	_tsplitpath(moduleName,drive,dir,NULL,NULL);

	String filename;
	filename.Format(_T("%s%s%s"),drive,dir,help.Get());

	DWORD error=(DWORD)::ShellExecute(GetHWND(),_T("open"),filename.Get(),NULL
		,filename.Get(),SW_SHOWDEFAULT);
	if(error<=32) {
		throw FileException(APPERR_CANT_OPEN_HELPFILE,filename.Get());
	}
}

//\ؑւ̃R}hIDpl̔ԍɂ
const static IDToValueTable s_panelTable[PANEL_COUNT+1]={
	{ID_VIEW_GENERAL,PANEL_GENERAL},
	{ID_VIEW_SPRAY,PANEL_SPRAY},
	{ID_VIEW_COLOR,PANEL_COLOR},
	{ID_VIEW_CATALOG,PANEL_CATALOG},

	{0,0}
};

//pl̕\ؑ
void AppFrame::OnViewPanel(int id)
{
	ToggleShowPanel(IDToValue(s_panelTable,id));
	OnUpdateWindow();
}

void AppFrame::OnUpdateViewPanel(int id,Command *command)
{
	bool isVisible=IsPanelVisible(IDToValue(s_panelTable,id));
	command->SetCheck(isVisible);
}

APP_END_NAMESPACE
