#include "prefix.h"

#include "catalogpanel.h"
#include "spray.h"
#include "spraystyle.h"
#include "main.h"
#include "string.h"
#include "winman.h"
#include "winmessage.h"
#include "file.h"

#include "fileexception.h"
#include "exception.h"
#include "error.h"
#include "showerror.h"

#include "registry.h"
#include "namedialog.h"
#include "filedialog.h"
#include "updatecontrol.h"
#include "messagebox.h"
#include "verify.h"
#include "gdiobject.h"
#include "idtovalue.h"
#include "presetdir.h"

#ifndef APP_STANDALONE
# include "pixia.h"
#endif

#include "resdialog.h"
#include "resbitmap.h"
#include "resstring.h"

#include <windows.h>
#include <memory.h>
#include <stdlib.h>
#include <tchar.h>

/*************************************************

	Xv[ꗗpl

**************************************************/

APP_BEGIN_NAMESPACE

#define FILE_HEADER	('P'+(('T'+(('C'+('L'<<8))<<8))<<8))

enum	//ꗗۑt@C̃o[W
{
	FILE_VERSION_0_1,	//ver0.1܂
	FILE_VERSION_0_2,	//ver0.2܂
	FILE_CURRENT_VERSION,	//݂̃o[W
};

//ݒt@Cǂ߂Ȃꍇ̋K̐ݒۑt@C̖O
#define PRESET_FNAME	"preset"

//t@C̊gq
//ύXꍇ̓\[XIDS_CATALOG_FILTERύX
#define FILE_EXT "spr"

static const IDToValueTable s_buttons[]={
	{IDC_CATALOG_ADD,IDB_CATALOG_ADD},
	{IDC_CATALOG_REPLACE,IDB_CATALOG_REPLACE},
	{IDC_CATALOG_REMOVE,IDB_CATALOG_REMOVE},
	{IDC_CATALOG_OPEN,IDB_CATALOG_OPEN},
};
#define BUTTON_COUNT (sizeof(s_buttons)/sizeof(IDToValueTable))

#define REGISTRYKEY "Panel\\Catalog"

class CatalogPanelMember
{
public:
	CatalogPanelMember()
		:isModified(false)
	{
	}

	String filename;	//ꗗۑt@C
	GdiObject buttonImage[BUTTON_COUNT];

	int initSelectIndex;	//ŏ̑I
	bool isModified;	//ꗗɕύX邩
};


CatalogPanel::CatalogPanel()
:Panel(_T(REGISTRYKEY),IDD_CATALOG)
,m_m(new CatalogPanelMember)
{
}

CatalogPanel::~CatalogPanel() throw()
{

}

//WXgƌ
void CatalogPanel::ExchangeSettings(Registry *registry)
{
	Panel::ExchangeSettings(registry);

	registry->OpenSubKey(_T(REGISTRYKEY));
	registry->Exchange(_T("File"),&(m_m->filename),NULL);
	registry->Exchange(_T("Select"),&m_m->initSelectIndex,-1);
}

//_CAȌ
void CatalogPanel::OnInitDialog()
{
	Panel::OnInitDialog();

	SetButtonImage(m_m->buttonImage,s_buttons,BUTTON_COUNT);

	bool isFileOpened=false;

	String filename;
	GetFileName(&filename);
	try {
		isFileOpened=Open(filename.Get());
	} catch(std::exception &e) {
		ShowError(NULL,e);
	}

	if(!isFileOpened) {
		GetPresetFileName(&filename);
		try {
			Open(filename.Get());
		} catch(std::exception &e) {
			ShowError(NULL,e);
		}
	}

	int curIndex=m_m->initSelectIndex;
	if(curIndex==LB_ERR) {
		curIndex=-1;
	}
	SendListMessage(LB_SETCURSEL,curIndex,0);
}

//Rg[ƕϐ̃f[^\̍XV
void CatalogPanel::DataExchange(UpdateControl *updater)
{
	bool isSelected=(GetSelect()!=LB_ERR);
	updater->Enable(IDC_CATALOG_REMOVE,isSelected);
	updater->Enable(IDC_CATALOG_REPLACE,isSelected);
}

//EBhEĵōڂSč폜
void CatalogPanel::OnDestroy()
{
	if(m_m->isModified) {
		try {
			String filename;
			GetFileName(&filename);
			Save(filename.Get());
		} catch(std::exception &e) {
			ShowError(NULL,e);
		}
	}
	m_m->initSelectIndex=GetSelect();
	RemoveAll();

	Panel::OnDestroy();
}

//ꗗۑt@C
void CatalogPanel::GetFileName(String *filename) const
{
	if(!m_m->filename.IsEmpty()) {
		*filename=m_m->filename;
		return;
	}

	_TCHAR moduleName[_MAX_PATH];
	::GetModuleFileName(GetInstance(),moduleName,_MAX_PATH);
	_TCHAR drive[_MAX_DRIVE];
	_TCHAR dir[_MAX_DIR];
	_TCHAR fname[_MAX_FNAME];
	_tsplitpath(moduleName,drive,dir,fname,NULL);

	String pixiaIniDir;
# ifndef APP_STANDALONE
	pixiaIniDir=PixiaInitializeDir();
# endif
	if(!pixiaIniDir.IsEmpty()) {
		_TCHAR pixiaDrive[_MAX_DRIVE];
		_TCHAR pixiaDir[_MAX_DIR];
		_tsplitpath(pixiaIniDir.Get(),pixiaDrive,pixiaDir,NULL,NULL);
		_tcscpy(drive,pixiaDrive);
		_tcscpy(dir,pixiaDir);
	}

	filename->Alloc(_MAX_PATH);
	_tmakepath(filename->GetW(),drive,dir,fname,_T(".") _T(FILE_EXT));
	filename->Shorten();
}

//t@Cǂ߂Ȃꍇ̊̐ݒ̃t@C
void CatalogPanel::GetPresetFileName(String *filename) const
{
	GetPresetDir(filename);
	filename->Cat(_T(PRESET_FNAME) _T(".") _T(FILE_EXT));
}

//t@C̃^O
//ォ珇ԂύXA菜肵Ă͂Ȃ
enum
{
	TAG_END,	//̒`̏I
	TAG_NAME,	//ڂ̖O

	TAG_REPULSION,
	TAG_GRAVITY,
	TAG_BRAKE,
	TAG_VISCOSITY,

	TAG_VELOCITY,
	TAG_COUNT,
	TAG_SPRAY_SIZE,
	TAG_LIFE,
	TAG_THROWING,

	TAG_LIMITED_SPEED,
	TAG_USE_LIMITED_SPEED,
	TAG_VELOCITY_WIGGLE,
	TAG_VELOCITY_WIDTH,
	TAG_VELOCITY_ROTATION_SPEED,
	TAG_VELOCITY_MODE,

	TAG_GRAVITY_AREA,
	TAG_REPULSION_AREA,
	TAG_VISCOSITY_AREA,

	TAG_SPIN_USE,
	TAG_SPIN_STEP,
	TAG_SPIN_ANGLE,
	TAG_SPIN_ANGLE_WIGGLE,

	TAG_SPLIT_USE,
	TAG_SPLIT_MODE,
	TAG_SPLIT_ANGLE1,
	TAG_SPLIT_ANGLE2,
	TAG_SPLIT_STEP1,
	TAG_SPLIT_STEP2,
	TAG_SPLIT_ANGLE_WIGGLE,
	TAG_SPLIT_LAG,
	TAG_SPLIT_GENERATIONS,
	TAG_SPLIT_RATE,

};

//t@Cǂݍ
//t@C݂ȂꍇfalseԂ
//̑̃G[͗O𓊂
bool CatalogPanel::Open(const _TCHAR * filename)
{
	File file;
	if(file.Open(filename)!=ERROR_SUCCESS) {
		return false;
	}

	int header=file.ReadRaw();
	if(header!=FILE_HEADER) {
		throw FileException(APPERR_INVALID_FILE,filename);
	}

	int version=file.ReadRaw();

	if(version>FILE_CURRENT_VERSION) {
		throw FileException(APPERR_CANT_READ_NEW_FILE,filename);
	}

	const int count=file.ReadRaw();
	FOR(int i=0;i<count;++i) {
		ReadStyle(&file,filename);
	}

	file.Close();

	m_m->isModified=false;
	UpdateData(UPDATE_CONTROL);

	return true;
}

//t@C獀ڂǂ
void CatalogPanel::ReadStyle(File *file,const _TCHAR * filename)
{
	String name;
	VERIFY(name.LoadString(IDS_CATALOG_NO_NAME));

	SPRAYSTYLE sprayStyle;
	InitializeSprayStyle(&sprayStyle);

	for(;;) {
		int tag=file->ReadRaw();
		switch(tag) {
		case TAG_END:
			CorrectSprayStyle(&sprayStyle);
			AddItem(name.Get(),sprayStyle);
			return;

		case TAG_NAME:
			file->ReadMultiByte(&name);
			break;

//^OƂɐU蕪ăt@Cǂݎ
#define READ_BY_CASE(a_varName,a_idName,a3,a4,a5,a6,a7,a8)	\
		case TAG_##a_idName:	\
			file->Read(&(sprayStyle.a_varName));	\
			break;

		SPRAYSTYLE_DO_FUNC(READ_BY_CASE)

		default:
			throw FileException(APPERR_INVALID_FILE,filename);
		}
	}
}

//t@Cɕۑ
void CatalogPanel::Save(const _TCHAR * filename)
{
	File file;
	file.Save(filename);

	file.WriteRaw(FILE_HEADER);
	file.WriteRaw(FILE_CURRENT_VERSION);
	const int count=GetCount();
	file.WriteRaw(count);

	FOR(int i=0;i<count;++i) {
		WriteStyle(&file,i);
	}

	file.Close();

	m_m->isModified=false;
	UpdateData(UPDATE_CONTROL);
}

//ڂۑ
void CatalogPanel::WriteStyle(File *file,int index)
{
	String name;
	GetItemName(index,&name);

	file->Write(TAG_NAME,name.Get());

	const SPRAYSTYLE *style=GetItem(index);

	//t@Cɍڂۑ
#define WRITE_STYLE(a_varName,a_idName,a3,a4,a5,a6,a7,a8)	\
	file->Write(TAG_##a_idName,style->a_varName);

	SPRAYSTYLE_DO_FUNC(WRITE_STYLE)

	file->WriteRaw(TAG_END);
}

void CatalogPanel::OnCommand(int id,int notifyCode,HWND hwnd)
{
	switch(id) {
	case IDC_CATALOG_ADD:
		if(notifyCode==BN_CLICKED) {
			OnAdd();
		}
		break;

	case IDC_CATALOG_REPLACE:
		if(notifyCode==BN_CLICKED) {
			OnReplace();
		}
		break;

	case IDC_CATALOG_REMOVE:
		if(notifyCode==BN_CLICKED) {
			OnRemove();
		}
		break;

	case IDC_CATALOG_OPEN:
		if(notifyCode==BN_CLICKED) {
			OnOpen();
		}
		break;

	case IDC_CATALOG_LIST:
		if(notifyCode==LBN_SELCHANGE) {
			OnSelect();
		}
		break;

	default:
		Panel::OnCommand(id,notifyCode,hwnd);
		break;
	}
}

//IĂ郊Xg̔ԍ
int CatalogPanel::GetSelect()
{
	return SendListMessage(LB_GETCURSEL,0,0);
}

//ڐ
int CatalogPanel::GetCount()
{
	return SendListMessage(LB_GETCOUNT,0,0);
}

//ڂ̖O
void CatalogPanel::GetItemName(int index,String *name)
{
	int length=SendListMessage(LB_GETTEXTLEN,(WPARAM)index,0);
	if(length==LB_ERR) {
		throw Exception(APPERR_INVALID_LIST_INDEX);
	}
	name->Alloc(length);
	SendListMessage(LB_GETTEXT,(WPARAM)index
		,(LPARAM)(name->Get()));
	name->Shorten();
}

//ǉ
void CatalogPanel::OnAdd()
{
	const SPRAYSTYLE *style=GetSprayStyle();

	String name;
	VERIFY(name.LoadString(IDS_CATALOG_NO_NAME));
	NameDialog dialog;
	dialog.SetName(name.Get());
	if(dialog.DoModal(GetHWND())==IDOK) {
		name=dialog.GetName();

		int newSelection=AddItem(name.Get(),*style);

		SendListMessage(LB_SETCURSEL,newSelection,0);

		m_m->isModified=true;
		UpdateData(UPDATE_CONTROL);
	}
}

//폜
void CatalogPanel::OnRemove()
{
	int index=GetSelect();
	if(index!=LB_ERR) {
		RemoveItem(index);
		m_m->isModified=true;

		int newSelection=index;
		if(GetCount()>=newSelection) {
			newSelection=GetCount()-1;
		}
		SendListMessage(LB_SETCURSEL,newSelection,0);

		UpdateData(UPDATE_CONTROL);
	}
}

//u
void CatalogPanel::OnReplace()
{
	int index=GetSelect();
	if(index!=LB_ERR) {
		SPRAYSTYLE *itemStyle=GetItem(index);
		SPRAYSTYLE currentStyle;
		GetSprayStyle(&currentStyle);
		*itemStyle=currentStyle;

		m_m->isModified=true;
		UpdateData(UPDATE_CONTROL);
	}
}

//t@CI ΊJȂΐV
void CatalogPanel::OnOpen()
{
	String currentName;
	GetFileName(&currentName);

	FileDialog dialog(IDS_CATALOG_OPEN,IDS_CATALOG_FILTER
		,currentName.Get(),_T(FILE_EXT),0,GetHWND());

	if(!dialog.SaveFileName()) {
		return;
	}
	const _TCHAR * selectedName=dialog.GetFileName();
	if(_tcslen(selectedName)==0) return;

	bool saveSuccess=false;
	try {
		if(m_m->isModified) {
			Save(currentName.Get());
		}
		saveSuccess=true;

	} catch(std::exception &e) {
		ShowError(GetFrameHWND(),e);
	}

	bool doRemoveCurrent=true;
	if(!saveSuccess) {
		int result=AppMessageBox(GetHWND(),IDS_CATALOG_REMOVE_ALL
			,MB_YESNOCANCEL|MB_ICONQUESTION);
		switch(result) {
		case IDYES:
			break;

		case IDNO:
			doRemoveCurrent=false;
			break;

		case IDCANCEL:
			return;
		}
	}

	if(doRemoveCurrent) {
		RemoveAll();
	}

	bool reload=saveSuccess;
	try {
		if(!Open(selectedName)) {
			Save(selectedName);
		}
		m_m->filename=selectedName;
		reload=false;
	} catch(std::exception &e) {
		ShowError(GetFrameHWND(),e);
	}

	//̈ꗗ̕ۑɂ͐Vꗗ̓ǂݍ݂ɂ͎sꍇ
	//̈ꗗǂݒ
	if(reload) {
		Open(currentName.Get());
	}

	//vύX@ڂ̊Ǘʓ|Ȃ̂ŃRg[ɔCĂ܂A
	//ꂽoAʓ|ȂƂɂȂĂ܂B
}

//I
void CatalogPanel::OnSelect()
{
	int index=GetSelect();
	if(index!=LB_ERR) {
		const SPRAYSTYLE *itemStyle=GetItem(index);	
		SetSprayStyle(*itemStyle);
		SendPanelMessage(PANEL_SPRAY,APP_WM_CHANGE_SPRAY_STYLE,0,0);
		UpdateData(UPDATE_CONTROL);
	}
}

//ڂɒǉ
int CatalogPanel::AddItem(const _TCHAR * name,const SPRAYSTYLE &sprayStyle)
{
	std::auto_ptr<SPRAYSTYLE> newStyle(new SPRAYSTYLE);
	*(newStyle.get())=sprayStyle;
	int index=SendListMessage(LB_ADDSTRING,0,(LPARAM)name);	
	if(index==LB_ERR || index==LB_ERRSPACE) {
		throw Exception(APPERR_CANT_ADD_LIST);
	}
	SendListMessage(LB_SETITEMDATA,index,(LPARAM)newStyle.get());
	newStyle.release();
	return index;
}

//ڂ폜
void CatalogPanel::RemoveItem(int index)
{
	SPRAYSTYLE *sprayStyle=GetItem(index);
	delete sprayStyle;
	
	SendListMessage(LB_DELETESTRING,index,0);
}

//SĂ̍ڂ폜
void CatalogPanel::RemoveAll()
{
	const int count=GetCount();
	if(count!=LB_ERR) {
		FOR(int i=0;i<count;++i) {
			RemoveItem(count-i-1);
		}
	}
}

//ڂ̃Xv[̏Ԃ𓾂
SPRAYSTYLE *CatalogPanel::GetItem(int index)
{
	return (SPRAYSTYLE*)SendListMessage(LB_GETITEMDATA,index,0);
}

//XgRg[ɃbZ[W𑗂
LONG CatalogPanel::SendListMessage(UINT message
	,WPARAM wparam,LPARAM lparam)
{
	return SendDlgItemMessage(IDC_CATALOG_LIST,message,wparam,lparam);
}

APP_END_NAMESPACE
