#include "prefix.h"

#include "dc.h"

#include <windows.h>
#include <assert.h>

#include "gdiobject.h"
#include "exception.h"

/*************************************************

	foCXReLXg̊Ǘ

**************************************************/

APP_BEGIN_NAMESPACE

DC::DC(HDC hdc)
	:m_hdc(hdc)
{
	Initialize();
}

DC::~DC() throw()
{
	Release();
	assert(m_hdc==NULL);
	assert(m_penObject==NULL);
	assert(m_brushObject==NULL);
	assert(m_bitmapObject==NULL);

	assert(m_oldPen==NULL);
	assert(m_oldBrush==NULL);
	assert(m_oldBitmap==NULL);
}

HDC DC::Get()
{
	assert(m_hdc!=NULL);
	return m_hdc;
}

//
void DC::Initialize()
{
	m_penObject=m_brushObject=m_bitmapObject=NULL;
	m_oldPen=m_oldBrush=m_oldBitmap=NULL;
	m_oldROP=m_oldBkMode=0;
}

//IuWFNgfoCXReLXg؂藣
void DC::Release() throw()
{
	if(m_hdc!=NULL) {
		DeselectObject(OBJ_PEN);
		DeselectObject(OBJ_BRUSH);
		DeselectObject(OBJ_BITMAP);

		if(m_oldROP!=0) {
			::SetROP2(m_hdc,m_oldROP);
			m_oldROP=0;
		}

		if(m_oldBkMode!=0) {
			::SetBkMode(m_hdc,m_oldBkMode);
			m_oldBkMode=0;
		}
	}
}

//objectI
void DC::SelectObject(GdiObject *object)
{
	assert(object!=NULL);

	DWORD type=object->Type();
	DeselectObject(type);

	HGDIOBJ oldObject(::SelectObject(m_hdc,object->Get()));
	assert(oldObject!=NULL);

	switch(type) {
	case OBJ_BITMAP:
		m_bitmapObject=object;
		m_oldBitmap=oldObject;
		break;

	case OBJ_PEN:
		m_penObject=object;
		m_oldPen=oldObject;
		break;

	case OBJ_BRUSH:
		m_brushObject=object;
		m_oldBrush=oldObject;
		break;

	default:
		assert(false);
	}

	object->Select(this);
}

//type̎ނ̃IuWFNgO
void DC::DeselectObject(DWORD type) throw()
{
	HGDIOBJ oldObject;
	GdiObject *object;
	switch(type) {
	case OBJ_BITMAP:
		oldObject=m_oldBitmap;
		m_oldBitmap=NULL;
		object=m_bitmapObject;
		m_bitmapObject=NULL;
		break;

	case OBJ_PEN:
		oldObject=m_oldPen;
		m_oldPen=NULL;
		object=m_penObject;
		m_penObject=NULL;
		break;

	case OBJ_BRUSH:
		oldObject=m_oldBrush;
		m_oldBrush=NULL;
		object=m_brushObject;
		m_brushObject=NULL;
		break;

	default:
		assert(false);
	}
	if(object!=NULL) {
		object->Deselect();
		assert(oldObject!=NULL);
#ifdef NDEBUG
		::SelectObject(m_hdc,oldObject);
#else
		HGDIOBJ obj(::SelectObject(m_hdc,oldObject));
		assert(obj!=(HGDIOBJ)GDI_ERROR);
#endif
	}
}

void DC::SetROP2(int rop)
{
	int lastROP=::SetROP2(m_hdc,rop);
	if(m_oldROP==0) {
		m_oldROP=lastROP;
	}
}

void DC::SetBkMode(int bkMode)
{
	int lastBkMode=::SetBkMode(m_hdc,bkMode);
	if(m_oldBkMode==0) {
		m_oldBkMode=lastBkMode;
	}
}

////////////////PaintDC////////////////////
PaintDC::PaintDC(HWND hwnd)
	:m_hwnd(hwnd)
{
	assert(hwnd!=NULL && ::IsWindow(hwnd));
	VERIFY(::BeginPaint(hwnd,&m_paintStruct)!=NULL);
	m_hdc=m_paintStruct.hdc;
}

PaintDC::~PaintDC() throw()
{
	DC::Release();
	assert(m_hwnd!=NULL && m_hdc!=NULL);
	::EndPaint(m_hwnd,&m_paintStruct);
	m_hdc=NULL;
}

////////////////ClientDC///////////////////
ClientDC::ClientDC(HWND hwnd)
	:m_hwnd(hwnd)
{
	assert(m_hwnd!=NULL && ::IsWindow(m_hwnd));
	m_hdc=::GetDC(hwnd);
	assert(m_hdc!=NULL);
}

ClientDC::~ClientDC() throw()
{
	DC::Release();
	assert(m_hwnd!=NULL && m_hdc!=NULL);
	VERIFY(::ReleaseDC(m_hwnd,m_hdc)!=0);
	m_hdc=NULL;
}

////////////////DrawItemDC/////////////////
DrawItemDC::DrawItemDC(HDC hdc)
	:DC(hdc)
{
	assert(m_hdc!=NULL);
}

DrawItemDC::~DrawItemDC() throw()
{
	DC::Release();
	assert(m_hdc!=NULL);
	m_hdc=NULL;
}

APP_END_NAMESPACE

