#ifndef APP_DELAY_BUFFER_H
#define APP_DELAY_BUFFER_H

#ifdef _MSC_VER
#pragma once
#endif

#include "site.h"
#include "delay.h"

/*************************************************

	obt@[̃f[^\ԂɎo

**************************************************/

APP_BEGIN_NAMESPACE

class DelayBuffer : public Delay
{
public:
	DelayBuffer(const void *buffer,int bytes)
		:m_buffer(buffer)
		,m_bytes(bytes)
		,m_pointer(0)
	{
	}

	virtual ~DelayBuffer() throw()
	{
	}

	virtual bool IsEOF() const
	{
		return m_pointer>=m_bytes;
	}

	virtual int PoolBytes() const
	{
		return m_bytes-m_pointer;
	}

	virtual const unsigned char *Get()
	{
		return static_cast<const unsigned char*>(m_buffer)+m_pointer;
	}

	virtual void Advance(int bytes)
	{
		assert(bytes<=PoolBytes());
		m_pointer+=bytes;
	}

	virtual int Read(void *data,int bytes)
	{
		int copyBytes=bytes;
		if(PoolBytes()<bytes) {
			copyBytes=PoolBytes();
		}
		memcpy(data,Get(),copyBytes);
		m_pointer+=copyBytes;
		return copyBytes;
	}
private:
	DelayBuffer(const DelayBuffer &source);
	const DelayBuffer &operator=(const DelayBuffer &source);

	const void *m_buffer;
	int m_bytes;
	int m_pointer;
};

APP_END_NAMESPACE

#endif
