#include "prefix.h"

#include "file.h"
#include "string.h"
#include "buffer.h"
#include "error.h"
#include "fileexception.h"

#include "fpoint.h"
#include "verify.h"

#include <windows.h>
#include <string.h>
#include <tchar.h>

/*************************************************

	t@C̓ǂݏ

**************************************************/

APP_BEGIN_NAMESPACE

class FileMember
{
public:
	FileMember()
		:handle(NULL)
		,isCreatedFile(false)
	{
	}
	String filename;
	HANDLE handle;	//t@Cnh
	bool isCreatedFile;	//ꂪt@C
};

File::File()
 :m_m(new FileMember)

{
}

File::~File() throw()
{
	if(m_m->handle!=NULL) {
		VERIFY(::CloseHandle(m_m->handle));
		if(m_m->isCreatedFile) {
			VERIFY(::DeleteFile(m_m->filename.Get()));
		}
	}
}

HANDLE File::Handle()
{
	return m_m->handle;
}

//ǂݎ̂߂ɊJ
DWORD File::Open(const _TCHAR * filename)
{
	assert(m_m->handle==NULL);
	m_m->filename=filename;

	HANDLE handle(::CreateFile(filename,GENERIC_READ
		,FILE_SHARE_READ,NULL,OPEN_EXISTING
		,FILE_ATTRIBUTE_NORMAL
		,NULL));

	if(handle==INVALID_HANDLE_VALUE) {
		return ::GetLastError();
	}

	m_m->handle=handle;
	return ERROR_SUCCESS;
}

//ۑ̂߂ɊJ
void File::Save(const _TCHAR * filename)
{
	assert(m_m->handle==NULL);
	m_m->filename=filename;

	HANDLE handle(::CreateFile(filename,GENERIC_WRITE
		,FILE_SHARE_WRITE,NULL,CREATE_NEW
		,FILE_ATTRIBUTE_NORMAL
		,NULL));

	if(handle==INVALID_HANDLE_VALUE) {
		handle=::CreateFile(filename,GENERIC_WRITE
			,FILE_SHARE_WRITE,NULL,CREATE_ALWAYS
			,FILE_ATTRIBUTE_NORMAL
			,NULL);
		if(handle==INVALID_HANDLE_VALUE) {
			throw FileException(APPERR_API,::GetLastError(),filename);
		}
	} else {
		m_m->isCreatedFile=true;
	}

	m_m->handle=handle;
}

//
void File::Close()
{
	m_m->isCreatedFile=false;
	if(m_m->handle!=NULL) {
		VERIFY(::CloseHandle(m_m->handle));
		m_m->handle=NULL;
	}
}

//w肵oCg
void File::WriteRaw(const void *data,int bytes)
{
	DWORD writtenBytes=0;
	if(!::WriteFile(m_m->handle,data,bytes,&writtenBytes,NULL)) {
		throw FileException(APPERR_API,::GetLastError(),m_m->filename.Get());
	}
}

//int
void File::WriteRaw(int value)
{
	WriteRaw(&value,sizeof(int));
}

//double
void File::WriteRaw(double value)
{
	WriteRaw(&value,sizeof(double));
}

//^Oint
void File::Write(int tag,int value)
{
	WriteRaw(tag);
	WriteRaw(static_cast<int>(sizeof(int)));
	WriteRaw(value);
}

//^Odouble
void File::Write(int tag,double value)
{
	WriteRaw(tag);
	WriteRaw(static_cast<int>(sizeof(double)));
	WriteRaw(value);
}

//^OƍW
void File::Write(int tag,const FPOINT &point)
{
	WriteRaw(tag);
	WriteRaw(static_cast<int>(sizeof(double)*2));
	WriteRaw(point.x);
	WriteRaw(point.y);
}

//^Oƕ
void File::Write(int tag,const _TCHAR * text)
{
	const char *mbTextPtr;
#ifdef _UNICODE
	String sText=text;
	int mbLength=sText.MultiByteLength()+1;
	Buffer<char> mbText;
	mbText.Alloc(mbLength);
	sText.ToMultiByte(mbText.GetW(),mbLength);
	mbTextPtr=mbText.Get();
#else
	mbTextPtr=text;	
#endif

	WriteRaw(tag);
	int textBytes=strlen(mbTextPtr)+1;
	WriteRaw(textBytes);
	WriteRaw(mbTextPtr,textBytes);
}

//^Obooll
void File::Write(int tag,bool flag)
{
	Write(tag,flag?TRUE:FALSE);
}

//w肵oCgǂ
void File::ReadRaw(void *data,int bytes)
{
	DWORD readBytes=0;
	if(!::ReadFile(m_m->handle,data,bytes,&readBytes,NULL)) {
		throw FileException(APPERR_API,::GetLastError(),m_m->filename.Get());
	}
}

//t@CPǂ
char File::ReadChar()
{
	char value;
	ReadRaw(&value,sizeof(char));
	return value;
}

//intǂ
int File::ReadRaw()
{
	int value;
	ReadRaw(&value,sizeof(int));
	return value;
}

//傫mFăf[^ǂ
void File::Read(void *data,int bytes)
{
	int sectionBytes=ReadRaw();
	if(sectionBytes!=bytes) {
		throw FileException(APPERR_INVALID_FILE,m_m->filename.Get());
	}
	ReadRaw(data,bytes);
}

//傫mFintǂ 
void File::Read(int *value)
{
	Read(value,sizeof(int));
}

//傫mFdoubleǂ
void File::Read(double *value)
{
	Read(value,sizeof(double));
}

//傫mFčWǂ
void File::Read(FPOINT *point)
{
	int sectionBytes=ReadRaw();
	if(sectionBytes!=sizeof(double)*2) {
		throw FileException(APPERR_INVALID_FILE,m_m->filename.Get());
	}
	ReadRaw(&(point->x),sizeof(double));
	ReadRaw(&(point->y),sizeof(double));
}

//ǂ
void File::ReadMultiByte(String *string)
{
	int sectionBytes=ReadRaw();
#ifdef _UNICODE
	Buffer<char> mbStr;
	mbStr.Alloc(sectionBytes);
	ReadRaw(mbStr.GetW(),sectionBytes);
	*string=mbStr.Get();
#else
	string->Alloc(sectionBytes);
	ReadRaw(string->GetW(),sectionBytes);
	string->Shorten();
#endif
}

//ǂ
void File::ReadUnicode(String *string)
{
	int sectionBytes=ReadRaw();
	int length=sectionBytes/sizeof(wchar_t);
#ifdef _UNICODE
	string->Alloc(length);
	ReadRaw(string->GetW(),sectionBytes);
	string->Shorten();
#else
	Buffer<wchar_t> unicodeStr;
	unicodeStr.Alloc(length);
	ReadRaw(unicodeStr.GetW(),sectionBytes);
	*string=unicodeStr.Get();
#endif
}

//boollǂ
void File::Read(bool *flag)
{
	int value;
	Read(&value);
	*flag=(value!=FALSE);
}

APP_END_NAMESPACE
