#include "prefix.h"

#include "Historyitem.h"

#include "exception.h"
#include "error.h"
#include "ptrdeque.h"
#include "ucharbuffer.h"
#include "delay.h"
#include "delaycompress.h"
#include "delaybuffer.h"

/*************************************************

	̍

**************************************************/

APP_BEGIN_NAMESPACE

#define ALLOC_BLOCK 1024

typedef PtrDeque<UCharBuffer> DataList;

class HistoryItemMember
{
public:
	HistoryItemMember()
		:key(0)
		,subKey(0)
		,dataListOffset(0)
	{
	}
	~HistoryItemMember()
	{
	}
	DataList dataList;
	UCharBuffer diff;	//
	int key;	//ނ̂߂̌
	int subKey;	//ނ̂߂̌2
	int dataListOffset;	//dataList̐擪̏
};

HistoryItem::HistoryItem() throw()
:m_m(new HistoryItemMember)
{
}

HistoryItem::~HistoryItem() throw()
{
}

void HistoryItem::SetKey(int key,int subKey)
{
	m_m->key=key;
	m_m->subKey=subKey;
}

int HistoryItem::GetKey() const
{
	return m_m->key;
}

int HistoryItem::GetSubKey() const
{
	return m_m->subKey;
}

int HistoryItem::Size() const
{
	return m_m->diff.Bytes();
}

enum
{
	TAG_INTEGER,
	TAG_DOUBLE,
	TAG_DATA,
};

void HistoryItem::WriteInteger(int *pointer,int value)
{
	WriteTag(pointer,TAG_INTEGER,&value,sizeof(int));
}

void HistoryItem::WriteDouble(int *pointer,double value)
{
	WriteTag(pointer,TAG_DOUBLE,&value,sizeof(double));
}

void HistoryItem::WriteData(int *pointer,Delay *delay)
{
	DelayCompress comp(delay,true);
	std::auto_ptr<UCharBuffer> buffer(new UCharBuffer);
	comp.Read(buffer.get());
	int index=m_m->dataList.Count()+m_m->dataListOffset;
	m_m->dataList.Add(buffer.get());
	buffer.release();
	WriteTag(pointer,TAG_DATA,&index,sizeof(int));
}

void HistoryItem::WriteTag(int *pointer,int tag,const void *data,int size)
{
	m_m->diff.Increase((*pointer+size+sizeof(int)+ALLOC_BLOCK)&~(ALLOC_BLOCK-1));
	memcpy(m_m->diff.GetW()+*pointer,&tag,sizeof(int));
	memcpy(m_m->diff.GetW()+*pointer+sizeof(int),data,size);
	*pointer+=size+sizeof(int);
}

void HistoryItem::Shorten(int size)
{
	m_m->diff.Realloc(size);
}

int HistoryItem::ReadInteger(int *pointer) const
{
	int value;
	ReadTag(pointer,TAG_INTEGER,&value,sizeof(int));
	return value;
}

double HistoryItem::ReadDouble(int *pointer) const
{
	double value;
	ReadTag(pointer,TAG_DOUBLE,&value,sizeof(double));
	return value;
}

void HistoryItem::ReadData(int *pointer,std::auto_ptr<Delay> *delay) const
{
	int index;
	ReadTag(pointer,TAG_DATA,&index,sizeof(int));
	index-=m_m->dataListOffset;
	const UCharBuffer *comp=m_m->dataList.At(index);

	std::auto_ptr<Delay> delayComp(
		new DelayBuffer(comp->Get(),comp->Bytes()));

	APP_RESET_AUTO_PTR((*delay),Delay
		,new DelayCompress(delayComp,false));
}

void HistoryItem::ReadTag(int *pointer,int tag,void *data,int size) const
{
	assert(*reinterpret_cast<const int*>
		(m_m->diff.Get()+*pointer)==tag);

	memcpy(data,m_m->diff.GetW()+*pointer+sizeof(int),size);
	*pointer+=size+sizeof(int);
}

APP_END_NAMESPACE
