#include "prefix.h"

#ifndef APP_STANDALONE

#include "string.h"
#include "image.h"
#include "channel.h"
#include "ucharbuffer.h"
#include "main.h"
#include "verify.h"

#include "resstring.h"

#include <windows.h>
#include <assert.h>

/*************************************************

	PIXIAp̃GNX|[g֐

**************************************************/

APP_BEGIN_NAMESPACE

//摜̏
struct LAYERINFO 
{
	int width;
	int height;
	int colorBits;
	char reserved[128];
};

//sǂݍ
struct GETLINEFUNC
{
	BOOL (*GetLine)(int y,int xStart,int xEnd,LPBYTE buffer,int layer);
	BOOL (*GetLineAlpha)(int y,int xStart,int xEnd,LPBYTE buffer,int layer,int alphaChannel);
	BOOL (*GetPalette)(BYTE *palette,int index);
	char reserved[64];
};

//so
struct SETLINEFUNC
{
	BOOL (*SetLine)(int y,int xStart,int xEnd,LPBYTE buffer,int layer);
	BOOL (*SetLineAlpha)(int y,int xStart,int xEnd,LPBYTE buffer,int layer,int alphaChannel);
	BOOL (*SetPalette)(BYTE *palette,int index);
	char reserved[64];
};

enum 
{
	PIXIA_BLUE,
	PIXIA_GREEN,
	PIXIA_RED
};

extern "C"
BOOL APP_API FilterGetTitle(HWND,LPSTR info
	,BOOL *hasOptionDialog,int *version)
{
	String plugInTitle;
	VERIFY(plugInTitle.LoadString(IDS_APPTITLE));

//****infoobt@̒킩܂ 30ɈӖ͂Ȃ****
	assert(plugInTitle.Length()<30);

#ifdef _UNICODE
	plugInTitle.ToMultiByte(info,30);
#else
	_mbscpy(reinterpret_cast<unsigned char *>(info)
		,reinterpret_cast<const unsigned char*>(plugInTitle.Get()));
#endif

	*hasOptionDialog=FALSE;
	*version=10;

	return TRUE;
}

static const LAYERINFO *s_layer=NULL;
static const GETLINEFUNC *s_getFunc=NULL;
static const SETLINEFUNC *s_setFunc=NULL;

static const char *s_initializeDir=NULL;

extern "C"
BOOL APP_API FilterProc(HWND hwnd,LAYERINFO *layer
	,SETLINEFUNC *setFunc,GETLINEFUNC *getFunc
	,LPSTR dir,LPBYTE)
{
	if(layer==NULL || layer->colorBits!=24
		||setFunc==NULL
		||getFunc==NULL) {
		return FALSE;
	}

	s_initializeDir=dir;

	s_layer=layer;
	s_getFunc=getFunc;
	s_setFunc=setFunc;

	MainLoop(hwnd);
	return TRUE;
}

SIZE PixiaSize()
{
	assert(s_layer!=NULL);
	SIZE size;
	size.cx=s_layer->width;
	size.cy=s_layer->height;
	return size;
}

const char * PixiaInitializeDir()
{
	return s_initializeDir;
}

#define PIXIA_OPACITY 1

void PixiaGetImage(Image *image)
{
	const SIZE size=PixiaSize();
	image->Create(size,PIXEL_BYTES);

	unsigned char *imageptr=image->GetW();
	UCharBuffer colorLine;
	colorLine.Alloc(size.cx*COLOR_BYTES+sizeof(LONG));
	UCharBuffer alphaLine;
	alphaLine.Alloc(size.cx+sizeof(LONG));
	FOR(int y=0;y<size.cy;++y) {
		(*(s_getFunc->GetLine))(y,0,size.cx,colorLine.GetW(),-1);
		(*(s_getFunc->GetLineAlpha))(y,0,size.cx,alphaLine.GetW(),-1,PIXIA_OPACITY);

		unsigned char *colorptr=colorLine.GetW();
		unsigned char *alphaptr=alphaLine.GetW();
		FOR(int x=0;x<size.cx;++x) {
			imageptr[CHANNEL_RED]=colorptr[PIXIA_RED];
			imageptr[CHANNEL_GREEN]=colorptr[PIXIA_GREEN];
			imageptr[CHANNEL_BLUE]=colorptr[PIXIA_BLUE];
			colorptr+=COLOR_BYTES;
			imageptr[CHANNEL_OPACITY]=*alphaptr;
			imageptr+=PIXEL_BYTES;
			++alphaptr;
		}
	}
}

void PixiaSetImage(const Image *image)
{
	const SIZE size=image->Size();

	const unsigned char *imageptr=image->Get();
	UCharBuffer colorLine;
	colorLine.Alloc(size.cx*COLOR_BYTES+sizeof(LONG));
	UCharBuffer alphaLine;
	alphaLine.Alloc(size.cx+sizeof(LONG));
	FOR(int y=0;y<size.cy;++y) {
		unsigned char *colorptr=colorLine.GetW();
		unsigned char *alphaptr=alphaLine.GetW();
		FOR(int x=0;x<size.cx;++x) {
			colorptr[PIXIA_RED]=imageptr[CHANNEL_RED];
			colorptr[PIXIA_GREEN]=imageptr[CHANNEL_GREEN];
			colorptr[PIXIA_BLUE]=imageptr[CHANNEL_BLUE];
			colorptr+=COLOR_BYTES;
			*alphaptr=imageptr[CHANNEL_OPACITY];
			++alphaptr;
			imageptr+=PIXEL_BYTES;
		}
		(*(s_setFunc->SetLine))(y,0,size.cx,colorLine.GetW(),-1);
		(*(s_setFunc->SetLineAlpha))(y,0,size.cx,alphaLine.GetW(),-1,PIXIA_OPACITY);
	}
}

APP_END_NAMESPACE

#endif //!APP_STANDALONE
