#include "prefix.h"

#include "registry.h"
#include "string.h"

#include "exception.h"
#include "fileexception.h"
#include "error.h"

#include "main.h"
#include "fpoint.h"

#ifndef APP_STANDALONE
#include "pixia.h"
#endif

#include <windows.h>
#include <math.h>
#include <tchar.h>

/*************************************************

	WXg[t@CƏݒ̌

**************************************************/

APP_BEGIN_NAMESPACE

/*WXg[̃L[
HKEY_CURRENT_USER̉	*/
#define APP_REGISTRY_KEY "Software\\USISI\\Painticle"

class RegistryMember
{
public:
	RegistryMember()
		:doesSetDefault(false)
		,useRegistry(false)
		,appKey(NULL)
		,subKey(NULL)
	{
	}

	String iniFile;
	String section;

	HKEY appKey;
	HKEY subKey;

	bool isReading;	//ǂݎ蒆
	bool doesSetDefault;	//lݒ肷邾
	bool useRegistry;
};

Registry::Registry()
	:m_m(new RegistryMember)
{

}

Registry::~Registry() throw()
{
	if(m_m->useRegistry) {
		CloseSubKey();
		if(m_m->appKey!=NULL) {
			::RegCloseKey(m_m->appKey);
		}
	}
}

//̒l̐ݒ肾
void Registry::DefaultMode()
{
	m_m->doesSetDefault=true;
}

//WXgɕۑ
void Registry::RegistryMode()
{
	m_m->useRegistry=true;
}

//AvP[ṼL[J
void Registry::OpenAppKey(bool isReading)
{
	m_m->isReading=isReading;

	_TCHAR moduleName[_MAX_PATH];
	::GetModuleFileName(GetInstance(),moduleName,_MAX_PATH);
	_TCHAR iniDrive[_MAX_DRIVE];
	_TCHAR iniDir[_MAX_DIR];
	_TCHAR iniFname[_MAX_FNAME];
	_tsplitpath(moduleName,iniDrive,iniDir,iniFname,NULL);

	if(!m_m->useRegistry) {
		String pixiaIniDir;
# ifndef APP_STANDALONE
		pixiaIniDir=PixiaInitializeDir();
# endif
		if(!pixiaIniDir.IsEmpty()) {
			_TCHAR pixiaDrive[_MAX_DRIVE];
			_TCHAR pixiaDir[_MAX_DIR];
			_tsplitpath(pixiaIniDir.Get(),pixiaDrive,pixiaDir,NULL,NULL);
			_tcscpy(iniDrive,pixiaDrive);
			_tcscpy(iniDir,pixiaDir);
		}

		m_m->iniFile.Alloc(_MAX_PATH);
		_tmakepath(m_m->iniFile.GetW(),iniDrive,iniDir,iniFname,_T(".ini"));
		m_m->iniFile.Shorten();
	} else {
		m_m->appKey=CreateKey(isReading,HKEY_CURRENT_USER
			,_T(APP_REGISTRY_KEY));
	}
}

//L[J쐬
HKEY Registry::CreateKey(bool isReading,HKEY openKey,const _TCHAR * key) const
{
	assert(m_m->useRegistry);

	DWORD mask;
	if(isReading) {
		mask=KEY_READ;
	} else {
		mask=KEY_WRITE;
	}
	HKEY resultKey=NULL;
	DWORD disposition;

	LONG regError=::RegCreateKeyEx(openKey,key
		,0,NULL,0,mask,NULL,&(resultKey),&disposition);
	if(regError!=ERROR_SUCCESS && !isReading) {
		throw Exception(APPERR_API,regError);
	}

	return resultKey;
}

//TuL[J
void Registry::OpenSubKey(const _TCHAR * subKey)
{
	if(!m_m->useRegistry) {
		if(subKey!=NULL) {
			m_m->section=subKey;
		} else {
			m_m->section=_T("App");
		}
	} else {
		if(m_m->appKey==NULL) {
			return;
		}
		CloseSubKey();
		m_m->subKey=CreateKey(m_m->isReading,m_m->appKey,subKey);
	}
}

//TuL[
void Registry::CloseSubKey()
{
	assert(m_m->useRegistry);

	if(m_m->subKey!=NULL) {
		::RegCloseKey(m_m->subKey);
		m_m->subKey=NULL;
	}
}

//̌
void Registry::Exchange(const _TCHAR * name,int *var,int defaultValue)
{
	if(m_m->isReading && m_m->doesSetDefault) {
		*var=defaultValue;
		return;
	}

	if(!m_m->useRegistry) {
		assert(!m_m->section.IsEmpty());
		if(m_m->isReading) {
			*var=::GetPrivateProfileInt(m_m->section.Get(),name
				,defaultValue,m_m->iniFile.Get());
		} else {
			String varString;
			varString.Format(_T("%d"),*var);
			if(!::WritePrivateProfileString(m_m->section.Get(),name
				,varString.Get(),m_m->iniFile.Get())) {
				throw FileException(APPERR_API,::GetLastError(),m_m->iniFile.Get());
			}
		}
	} else {
		if(m_m->isReading) {
			DWORD valueDWord;
			DWORD size=sizeof(DWORD);
			if(m_m->subKey!=NULL
				&& ::RegQueryValueEx(m_m->subKey,name,NULL,NULL
				 ,(BYTE*)&valueDWord,&size)==ERROR_SUCCESS) {

				*var=valueDWord;
			} else {
				*var=defaultValue;
			}
		} else {
			DWORD valueDWord=*var;
			LONG regError=::RegSetValueEx(m_m->subKey,name,0
				,REG_DWORD,(BYTE*)&valueDWord,sizeof(DWORD));
			if(regError!=ERROR_SUCCESS) {
				throw Exception(APPERR_API,regError);
			}
		}
	}
}

//booľ
void Registry::Exchange(const _TCHAR * name,bool *var,bool defaultFlag)
{
	int varInt=(*var)?TRUE:FALSE;
	Exchange(name,&varInt,defaultFlag);
	if(m_m->isReading) {
		*var=varInt!=FALSE;
	}
}

//unsigned chař
void Registry::Exchange(const _TCHAR * name,unsigned char *var,unsigned char defaultValue)
{
	int varInt=*var;
	Exchange(name,&varInt,defaultValue);
	if(m_m->isReading) {
		*var=varInt;
	}
}

//̌
void Registry::Exchange(const _TCHAR * name,double *var,double defaultValue)
{
	if(m_m->isReading && m_m->doesSetDefault) {
		*var=defaultValue;
		return;
	}
	
	String varString;
	String defaultString;
	if(!m_m->isReading) {
		varString.Format(_T("%g"),*var);
	} else {
		defaultString.Format(_T("%g"),defaultString.Get());
	}

	Exchange(name,&varString,defaultString.Get());

	if(m_m->isReading) {
		*var=_tcstod(varString.Get(),NULL);
	}
}

//̌
void Registry::Exchange(const _TCHAR * name,String *var,const _TCHAR * defaultText)
{
	if(m_m->isReading && m_m->doesSetDefault) {
		*var=defaultText;
		return;
	}

	if(!m_m->useRegistry) {
		assert(!m_m->section.IsEmpty());
		if(m_m->isReading) {
			int length=0;
			do {
				length+=1024;
				var->Alloc(length);
				if(::GetPrivateProfileString(m_m->section.Get()
					,name,defaultText,var->GetW(),length
					,m_m->iniFile.Get())==0) {
					var->Free();
					break;
				}
			} while(var->Length()>=length-1);
			var->Shorten();
		} else {
			if(!::WritePrivateProfileString(m_m->section.Get(),name
				,var->Get(),m_m->iniFile.Get())) {
				throw FileException(APPERR_API,::GetLastError(),m_m->iniFile.Get());
			}
		}
	} else {
		if(m_m->isReading) {
			DWORD size=0;
			if(m_m->subKey!=NULL
				&& ::RegQueryValueEx(m_m->subKey,name,NULL,NULL
				 ,NULL,&size)==ERROR_SUCCESS) {
				var->Alloc(size);
				::RegQueryValueEx(m_m->subKey,name,NULL,NULL
					,(BYTE*)(var->Get()),&size);
			} else {
				*var=defaultText;
			}
		} else {
			const _TCHAR * varText=var->Get();
			LONG regError=::RegSetValueEx(m_m->subKey,name,0,REG_SZ,(const BYTE*)(varText)
				,var->Length()+1);
			if(regError!=ERROR_SUCCESS) {
				throw Exception(APPERR_API,regError);
			}
		}
	}
}

#define EXCHANGE_PART(a_partString,a_varPart,a_defaultValue)	\
	{	\
		String subKeyPart;	\
		subKeyPart.Format(_T("%s%s"),name,a_partString);	\
		Exchange(subKeyPart.Get(),(int*)&(a_varPart),a_defaultValue);	\
	}

//`̌
void Registry::Exchange(const _TCHAR * name,RECT *var,const RECT &defaultRect)
{
	if(m_m->isReading && m_m->doesSetDefault) {
		*var=defaultRect;
		return;
	}

	EXCHANGE_PART(_T("Left"),var->left,defaultRect.left);
	EXCHANGE_PART(_T("Top"),var->top,defaultRect.top);
	EXCHANGE_PART(_T("Right"),var->right,defaultRect.right);
	EXCHANGE_PART(_T("Bottom"),var->bottom,defaultRect.bottom);
}

//_̍W
void Registry::Exchange(const _TCHAR * name,FPOINT *var,const FPOINT &defaultPoint)
{
	if(m_m->isReading && m_m->doesSetDefault) {
		*var=defaultPoint;
		return;
	}
	
	String subKeyPart;
	subKeyPart.Format(_T("%s%s"),name,_T("X"));
	Exchange(subKeyPart.Get(),&(var->x),defaultPoint.x);

	subKeyPart.Format(_T("%s%s"),name,_T("Y"));
	Exchange(subKeyPart.Get(),&(var->y),defaultPoint.y);
}

bool Registry::IsReading() const
{
	return m_m->isReading;
}

APP_END_NAMESPACE
