#include "prefix.h"

#include "splitdialog.h"
#include "updatecontrol.h"
#include "showerror.h"
#include "volumecontrol.h"
#include "anglecontrol.h"
#include "spray.h"
#include "spraystyle.h"
#include "idtovalue.h"
#include "particleinfo.h"
#include "gdiobject.h"

#include "resdialog.h"
#include "resstring.h"
#include "resbitmap.h"

/*********************************************

	̐ݒ̃_CAO

*********************************************/

APP_BEGIN_NAMESPACE

static const IDToValueTable s_buttons[]={
	{IDC_SPLIT_SINGLE,IDB_SPLIT_SINGLE},
	{IDC_SPLIT_FORKED,IDB_SPLIT_FORKED},
	{IDC_SPLIT_DOUBLE,IDB_SPLIT_DOUBLE},
};
#define BUTTON_COUNT (sizeof(s_buttons)/sizeof(IDToValueTable))

class SplitDialogMember
{
public:
	SplitDialogMember()
	{
	}

	AngleControl angle1Control;
	AngleControl angle2Control;
	VolumeControl angleWiggleControl;
	VolumeControl rateControl;

	GdiObject buttonImage[BUTTON_COUNT];
	
	double angle1;	//px1
	double angle2;	//px2
	double angleWiggle;	//px̗h炬
	double rate;	//m
	int mode;	//̂
	int step1;	//Ԋu1
	int step2;	//Ԋu2
	int lag;	//􎞊̂
	int generationsMax;	//̐㐔
	bool use;	//邩
};

SplitDialog::SplitDialog()
:Dialog(IDD_SPLIT)
,m_m(new SplitDialogMember)
{
}

SplitDialog::~SplitDialog() throw()
{
}

//
void SplitDialog::OnInitDialog()
{
	SetButtonImage(m_m->buttonImage,s_buttons,BUTTON_COUNT);

	m_m->angle1Control.SubclassDialogItem(
		IDC_SPLIT_ANGLE1,this);
	m_m->angle2Control.SubclassDialogItem(
		IDC_SPLIT_ANGLE2,this);
	m_m->angleWiggleControl.SubclassDialogItem(
		IDC_SPLIT_ANGLE_WIGGLE,this);
	m_m->rateControl.SubclassDialogItem(
		IDC_SPLIT_RATE,this);

	m_m->angleWiggleControl.SetRange(
		SprayStyleType(SPRAYSTYLE_SPLIT_ANGLE_WIGGLE)->lowerLimit
		,SprayStyleType(SPRAYSTYLE_SPLIT_ANGLE_WIGGLE)->upperLimit);
	m_m->rateControl.SetRange(
		SprayStyleType(SPRAYSTYLE_SPLIT_RATE)->lowerLimit
		,SprayStyleType(SPRAYSTYLE_SPLIT_RATE)->upperLimit);

	m_m->angle1Control.SetAsRotation(true);
	m_m->angle2Control.SetAsRotation(true);
	m_m->angleWiggleControl.SetShowMode(VOLUME_SHOW_DEGREES);
	m_m->rateControl.SetShowMode(VOLUME_SHOW_PERCENT);

	const SPRAYSTYLE *sprayStyle=GetSprayStyle();
	m_m->use=sprayStyle->useSplit;
	m_m->mode=sprayStyle->splitMode;
	m_m->step1=sprayStyle->splitStep1;
	m_m->step2=sprayStyle->splitStep2;
	m_m->angle1=sprayStyle->splitAngle1;
	m_m->angle2=sprayStyle->splitAngle2;
	m_m->angleWiggle=sprayStyle->splitAngleWiggle;
	m_m->lag=sprayStyle->splitLag;
	m_m->rate=sprayStyle->splitRate;
	m_m->generationsMax=sprayStyle->splitGenerationsMax;
}

void SplitDialog::OnCommand(int id,int notifyCode,HWND hwnd)
{
	switch(id) {
	case IDC_SPLIT_USE:
	case IDC_SPLIT_SINGLE:
	case IDC_SPLIT_FORKED:
	case IDC_SPLIT_DOUBLE:
		if(notifyCode==BN_CLICKED) {
			UpdateData(UPDATE_VAR);
			UpdateData(UPDATE_CONTROL);
		}
		break;

	default:
		Dialog::OnCommand(id,notifyCode,hwnd);
		break;
	}
}

static const IDToValueTable s_modeTable[]=
{
	{IDC_SPLIT_SINGLE,SPLIT_SINGLE},
	{IDC_SPLIT_FORKED,SPLIT_FORKED},
	{IDC_SPLIT_DOUBLE,SPLIT_DOUBLE},

	{0,0}
};

//Rg[ƕϐ̃f[^\̍XV
void SplitDialog::DataExchange(UpdateControl *updater)
{
	updater->Check(IDC_SPLIT_USE,&m_m->use);
	updater->RadioButton(s_modeTable,&m_m->mode);
	updater->Edit(IDC_SPLIT_STEP1,&m_m->step1);
	updater->Edit(IDC_SPLIT_STEP2,&m_m->step2);
	updater->Angle(IDC_SPLIT_ANGLE1,&m_m->angle1);
	updater->Angle(IDC_SPLIT_ANGLE2,&m_m->angle2);
	updater->Volume(IDC_SPLIT_ANGLE_WIGGLE,&m_m->angleWiggle);
	updater->Edit(IDC_SPLIT_LAG,&m_m->lag);
	updater->Edit(IDC_SPLIT_GENERATIONS,&m_m->generationsMax);
	updater->Volume(IDC_SPLIT_RATE,&m_m->rate);

	updater->Enable(IDC_SPLIT_SINGLE,m_m->use);
	updater->Enable(IDC_SPLIT_FORKED,m_m->use);
	updater->Enable(IDC_SPLIT_DOUBLE,m_m->use);
	updater->Enable(IDC_SPLIT_STEP1,m_m->use);
	updater->Enable(IDC_SPLIT_STEP2,m_m->use && m_m->mode==SPLIT_DOUBLE);
	updater->Enable(IDC_SPLIT_ANGLE1,m_m->use);
	updater->Enable(IDC_SPLIT_ANGLE2,m_m->use && m_m->mode!=SPLIT_SINGLE);
	updater->Enable(IDC_SPLIT_ANGLE_WIGGLE,m_m->use);
	updater->Enable(IDC_SPLIT_LAG,m_m->use && m_m->mode==SPLIT_DOUBLE);
	updater->Enable(IDC_SPLIT_GENERATIONS,m_m->use);
	updater->Enable(IDC_SPLIT_RATE,m_m->use);
}

//Kp
bool SplitDialog::Apply()
{
	SPRAYSTYLE sprayStyle;
	GetSprayStyle(&sprayStyle);

	int stepLower=SprayStyleType(SPRAYSTYLE_SPLIT_STEP1)->lowerLimit;
	if(m_m->step1<stepLower || sprayStyle.life<m_m->step1) {
		ShowError(GetHWND(),IDS_ERR_RANGE_SPLIT_STEP,stepLower);
		return false;
	}
	if(m_m->step2<stepLower || sprayStyle.life<m_m->step2) {
		ShowError(GetHWND(),IDS_ERR_RANGE_SPLIT_STEP,stepLower);
		return false;
	}
	int lagLower=SprayStyleType(SPRAYSTYLE_SPLIT_LAG)->lowerLimit;
	if(m_m->lag<lagLower || sprayStyle.life<m_m->lag) {
		ShowError(GetHWND(),IDS_ERR_RANGE_SPLIT_LAG,lagLower);
		return false;
	}
	int generationsLower=SprayStyleType(SPRAYSTYLE_SPLIT_GENERATIONS)->lowerLimit;
	if(m_m->generationsMax<generationsLower) {
		ShowError(GetHWND(),IDS_ERR_RANGE_SPLIT_GENERATIONS,generationsLower);
		return false;
	}

	sprayStyle.useSplit=m_m->use;
	sprayStyle.splitMode=m_m->mode;
	sprayStyle.splitStep1=m_m->step1;
	sprayStyle.splitStep2=m_m->step2;
	sprayStyle.splitAngle1=m_m->angle1;
	sprayStyle.splitAngle2=m_m->angle2;
	sprayStyle.splitAngleWiggle=m_m->angleWiggle;
	sprayStyle.splitLag=m_m->lag;
	sprayStyle.splitGenerationsMax=m_m->generationsMax;
	sprayStyle.splitRate=m_m->rate;
	SetSprayStyle(sprayStyle);
	return true;
}

APP_END_NAMESPACE
