#include "prefix.h"

#include "toolbar.h"
#include "main.h"
#include "exception.h"
#include "error.h"
#include "verify.h"

#include <windows.h>
#include <commctrl.h>

/*************************************************

	c[o[

**************************************************/

APP_BEGIN_NAMESPACE

HWND CreateToolBar(const struct ToolBarButton *buttons
	,int id,HWND parentWindow)
{
	HWND toolBar=::CreateWindowEx(
		0,TOOLBARCLASSNAME,NULL
		,WS_CHILD|TBSTYLE_TOOLTIPS|CCS_TOP
		,0,0,0,0,parentWindow,(HMENU)id,GetInstance()
		,NULL);

	if(toolBar==NULL) {
		throw Exception(APPERR_CANT_CREATE_WINDOW);
	}

	::SendMessage(toolBar,TB_BUTTONSTRUCTSIZE,(WPARAM)sizeof(TBBUTTON),0);

	for(int i=0,bitmapCount=0;buttons[i].commandId!=-1;i++) {
		if(buttons[i].bitmapId!=0) {
			TBADDBITMAP addBitmap;
			addBitmap.hInst=GetInstance();
			addBitmap.nID=buttons[i].bitmapId;
			VERIFY(::SendMessage(toolBar,TB_ADDBITMAP,(WPARAM)1,(LPARAM)&addBitmap)!=-1);
		}
			
		int commandId=buttons[i].commandId;

		TBBUTTON tbButton;
		if(buttons[i].bitmapId!=0) {
			tbButton.iBitmap=bitmapCount;
		} else {
			tbButton.iBitmap=0;
		}
		tbButton.idCommand=commandId;
		tbButton.fsState=buttons[i].state|TBSTATE_ENABLED;
		tbButton.fsStyle=buttons[i].style|TBSTYLE_TOOLTIPS;
		tbButton.dwData=0;
		tbButton.iString=i;
		::SendMessage(toolBar,TB_ADDBUTTONS,(WPARAM)1,(LPARAM)&tbButton);

		if(buttons[i].bitmapId!=0) {
			bitmapCount++;
		}
	}
	return toolBar;
}

void ToolBarButtonState(HWND toolBar,int id,int type,bool flag)
{
	int state=::SendMessage(toolBar,TB_GETSTATE,(WPARAM)id,0);

	state&=~type;
	if(flag) {
		state|=type;
	}

	::SendMessage(toolBar,TB_SETSTATE
		,(WPARAM)id
		,(LPARAM)MAKELONG(state,0));
}

//c[o[ĂԂ̐؂ւ
void ToolBarButtonCheck(HWND toolBar,int id,bool isChecked)
{
	ToolBarButtonState(toolBar,id,TBSTATE_CHECKED,isChecked);
}

//c[o[̎gpEs̐؂ւ
void ToolBarButtonEnable(HWND toolBar,int id,bool isEnabled)
{
	ToolBarButtonState(toolBar,id,TBSTATE_ENABLED,isEnabled);
}

APP_END_NAMESPACE
