#include "prefix.h"

#include "ucharbuffer.h"
#include "bufferholder.h"

#include <stdlib.h>
#include <memory>

/*************************************************

	[̊m

**************************************************/

APP_BEGIN_NAMESPACE

UCharBuffer::UCharBuffer()
:m_holder(NULL)
{
}

UCharBuffer::UCharBuffer(const UCharBuffer &source)
{
	*this=source;
}

UCharBuffer::~UCharBuffer() throw()
{
	Free();
}

const UCharBuffer& UCharBuffer::operator=(const UCharBuffer &source)
{
	Free();
	
	m_holder=source.m_holder;
	m_holder->IncreaseReferenceCount();

	return *this;
}

//obt@̊m
void UCharBuffer::Alloc(int bytes)
{
	assert(bytes>=0);

	Free();

	if(bytes>0) {
		m_holder=new BufferHolder();
		m_holder->Alloc(bytes);
	}
}

//Ƃ̃f[^\c܂܃obt@̑傫ς
void UCharBuffer::Realloc(int bytes)
{
	assert(bytes>=0);

	if(bytes==0) {
		Free();
	} else if(m_holder==NULL) {
		Alloc(bytes);
	} else if(m_holder->IsSharing()) {
		std::auto_ptr<BufferHolder> newHolder
			(new BufferHolder(*m_holder,bytes));
		m_holder->DecreaseReferenceCount();
		m_holder=newHolder.release();
	} else {
		m_holder->Realloc(bytes);
	}
}

//Vȑ傫݂̑傫傫ꍇobt@̑傫ς
void UCharBuffer::Increase(int bytes)
{
	assert(bytes>=0);
	if(m_holder==NULL) {
		Alloc(bytes);
	} else if(m_holder->Bytes()<bytes) {
		Realloc(bytes);
	}
}

//
void UCharBuffer::Free() throw()
{
	if(m_holder!=NULL) {
		m_holder->DecreaseReferenceCount();
		m_holder=NULL;
	}
}

//ǂݎ邽߂ɃAhX𓾂
const unsigned char *UCharBuffer::Get() const
{
	if(m_holder==NULL) {
		return NULL;
	}
	return m_holder->Get();
}

//ނ߂ɃAhX𓾂
unsigned char *UCharBuffer::GetW()
{
	if(m_holder==NULL) {
		return NULL;
	}
	if(m_holder->IsSharing()) {
		std::auto_ptr<BufferHolder> newHolder
			(new BufferHolder(*m_holder));
		m_holder->DecreaseReferenceCount();
		m_holder=newHolder.release();
	}
	return m_holder->Get();
}

//vf
//int UCharBuffer::Count() const
//{
//	return Bytes();
//}

//oCg
int UCharBuffer::Bytes() const
{
	if(m_holder==NULL) {
		return 0;
	}
	return m_holder->Bytes();
}

//obt@
bool UCharBuffer::IsEmpty() const
{
	return m_holder==NULL;
}

//obt@ւ
void UCharBuffer::Swap(UCharBuffer &other)
{
	BufferHolder *holder=other.m_holder;
	other.m_holder=m_holder;
	m_holder=holder;
}

APP_END_NAMESPACE
