#include "prefix.h"

#include "velocitydetail.h"
#include "updatecontrol.h"
#include "spray.h"
#include "spraystyle.h"
#include "idtovalue.h"
#include "showerror.h"
#include "gdiobject.h"
#include "volumecontrol.h"
#include "string.h"
#include "verify.h"

#include "resdialog.h"
#include "resstring.h"
#include "resbitmap.h"

#include <windows.h>

/*************************************************

	̏ڍאݒ_CAO

**************************************************/

APP_BEGIN_NAMESPACE

static const IDToValueTable s_buttons[]={
	{IDC_VELOCITY_LEFT_SPIN,IDB_VELOCITY_LEFT_SPIN},
	{IDC_VELOCITY_RIGHT_SPIN,IDB_VELOCITY_RIGHT_SPIN},
	{IDC_VELOCITY_WIPER,IDB_VELOCITY_WIPER},
	{IDC_VELOCITY_RANDOM,IDB_VELOCITY_RANDOM},
};
#define BUTTON_COUNT (sizeof(s_buttons)/sizeof(IDToValueTable))

class VelocityDetailMember
{
public:
	VolumeControl wiggleControl;
	GdiObject buttonImage[BUTTON_COUNT];	
	double wiggle;
	double width;
	double rotationSpeed;
	int mode;
};

VelocityDetail::VelocityDetail()
:LimitDialog(IDD_VELOCITY_DETAIL)
,m_m(new VelocityDetailMember)
{
}

VelocityDetail::~VelocityDetail() throw()
{
}

//_CAȌ
void VelocityDetail::OnInitDialog()
{
	LimitDialog::OnInitDialog();

	SetButtonImage(m_m->buttonImage,s_buttons,BUTTON_COUNT);

	m_m->wiggleControl.SubclassDialogItem(IDC_VELOCITY_WIGGLE,this);
	m_m->wiggleControl.SetShowMode(VOLUME_SHOW_PERCENT);

	const SPRAYSTYLE *sprayStyle=GetSprayStyle();
	m_m->wiggle=sprayStyle->velocityWiggle;
	m_m->width=sprayStyle->velocityWidth;
	m_m->rotationSpeed=sprayStyle->velocityRotationSpeed;
	m_m->mode=sprayStyle->velocityMode;
}

void VelocityDetail::OnCommand(int id,int notifyCode,HWND hwnd)
{
	switch(id) {
	case IDC_VELOCITY_LEFT_SPIN:
	case IDC_VELOCITY_RIGHT_SPIN:
	case IDC_VELOCITY_WIPER:
	case IDC_VELOCITY_RANDOM:
		if(notifyCode==BN_CLICKED) {
			UpdateData(UPDATE_VAR);
			UpdateData(UPDATE_CONTROL);
		}
		break;

	default:
		Dialog::OnCommand(id,notifyCode,hwnd);
		break;
	}
}

static const IDToValueTable s_modeTable[]=
{
	{IDC_VELOCITY_LEFT_SPIN,VELOCITY_LEFT_SPIN},
	{IDC_VELOCITY_RIGHT_SPIN,VELOCITY_RIGHT_SPIN},
	{IDC_VELOCITY_WIPER,VELOCITY_WIPER},
	{IDC_VELOCITY_RANDOM,VELOCITY_RANDOM},

	{0,0}
};

//Rg[ƕϐ̃f[^\̍XV
void VelocityDetail::DataExchange(UpdateControl *updater)
{
	LimitDialog::DataExchange(updater);

	updater->Volume(IDC_VELOCITY_WIGGLE,&m_m->wiggle);
	updater->Edit(IDC_VELOCITY_WIDTH,&m_m->width);
	updater->Edit(IDC_VELOCITY_ROTATION_SPEED,&m_m->rotationSpeed);
	updater->RadioButton(s_modeTable,&m_m->mode);

	updater->Enable(IDC_VELOCITY_ROTATION_SPEED
		,m_m->mode!=VELOCITY_RANDOM);
}

//{^
bool VelocityDetail::Apply()
{
	if(!LimitDialog::Apply()) {
		return false;
	}

	double widthLower=SprayStyleType(SPRAYSTYLE_VELOCITY_WIDTH)->lowerLimit;
	double widthUpper=SprayStyleType(SPRAYSTYLE_VELOCITY_WIDTH)->upperLimit;
	if(m_m->width<widthLower || widthUpper<m_m->width) {
		ShowError(GetHWND(),IDS_ERR_RANGE_VELOCITY_WIDTH,widthLower,widthUpper);
		return false;
	}

	double rotationLower=SprayStyleType(SPRAYSTYLE_VELOCITY_ROTATION_SPEED)->lowerLimit;
	double rotationUpper=SprayStyleType(SPRAYSTYLE_VELOCITY_ROTATION_SPEED)->upperLimit;
	if(m_m->rotationSpeed<rotationLower || rotationUpper<m_m->rotationSpeed) {
		ShowError(GetHWND(),IDS_ERR_RANGE_ROTATION_SPEED,rotationLower,rotationUpper);
		return false;
	}

	SPRAYSTYLE sprayStyle;
	GetSprayStyle(&sprayStyle);
	sprayStyle.velocityWiggle=m_m->wiggle;
	sprayStyle.velocityWidth=m_m->width;
	sprayStyle.velocityRotationSpeed=m_m->rotationSpeed;
	sprayStyle.velocityMode=m_m->mode;
	SetSprayStyle(sprayStyle);
	return true;
}

APP_END_NAMESPACE
