#ifndef APP_WINDOW_H
#define APP_WINDOW_H

#ifdef _MSC_VER
#pragma once
#endif

#include "site.h"

#include <windows.h>
#include <tchar.h>

/*************************************************

	EBhENX

**************************************************/

APP_BEGIN_NAMESPACE

class String;

class Window
{
public:
	Window();
	virtual ~Window() throw();

	void Create(const _TCHAR * caption,UINT style,const RECT &rect,HWND parentWindow);
	virtual LRESULT WindowProc(UINT msg,WPARAM wparam,LPARAM lparam);
	void Attach(HWND hwnd);
	void Detach();

	bool ShowWindow(int mode);
	virtual bool DestroyWindow();
	bool GetClientRect(RECT *rect) const;
	bool GetWindowRect(RECT *rect) const;
	HWND GetHWND() const;
	HWND GetParent() const;
	bool SetWindowPos(Window *wndInsertAfter,int x,int y,int cx,int cy,UINT flags);
	LONG SetWindowLong(int index,LONG dwNewLong);
	LONG GetWindowLong(int index) const;
	bool SetScrollInfo(int fnBar,SCROLLINFO *pSi,BOOL redraw);
	bool GetScrollInfo(int fnBar,SCROLLINFO *pSi) const;
	int ScrollWindowEx(int dx,int dy,const RECT *scroll
		,const RECT *clip,HRGN updateRegion
		,RECT *updateRect,UINT flags);
	bool UpdateWindow();
	LRESULT SendMessage(UINT message,WPARAM wparam,LPARAM lparam);
	bool PostMessage(UINT message,WPARAM wparam,LPARAM lparam);
	
	bool InvalidateRect(const RECT *rect=NULL,bool doesErase=true);
	bool ScreenToClient(POINT *point) const;
	bool ClientToScreen(POINT *point) const;
	HWND SetCapture();
	bool DragDetect(POINT point) const;
	bool SetWindowText(const _TCHAR * text);
	void GetWindowText(String *string) const;

	bool IsWindow() const throw();
	bool IsWindowEnabled() const;
	bool IsWindowVisible() const;

	HWND GetDlgItem(int itemID) const;
	bool SetDlgItemText(int itemID,const _TCHAR * text);
	void GetDlgItemText(int itemID,String *string) const;
	bool SetDlgItemInt(int itemID,int value);
	int GetDlgItemInt(int itemID,BOOL *succeed) const;
	bool CheckDlgButton(int itemID,UINT state);
	UINT IsDlgButtonChecked(int itemID) const;

	LONG SendDlgItemMessage(int itemID,UINT msg
		,WPARAM wparam,LPARAM lparam);
	LONG GetID() const;
	
	void SubclassDialogItem(UINT itemID,Window *parent);
	void SubclassWindow(HWND hwnd);
	void UnsubclassWindow();

protected:
	void RegisterWindowClass();
	virtual void OnDestroy();
	virtual void OnCommand(int id,int notifyCode,HWND hwnd);
	LRESULT OldProc(UINT msg,WPARAM wparam,LPARAM lparam);
private:
	//`Ȃ
	Window(const Window &source);
	const Window &operator=(const Window &source);

	HWND m_hwnd;
	WNDPROC m_oldProc;
};

inline const int Width(RECT rect)
{
	return rect.right-rect.left;
}

inline const int Height(RECT rect)
{
	return rect.bottom-rect.top;
}

Window *GetWindowObject(HWND hwnd);

APP_END_NAMESPACE

#endif

